require(['linesegment','game','class','event','room','vector','intrusivelist','dintrusivelist'], function(LineSegment,Game,Class,Event,Room,Vector,IntrusiveList,DIntrusiveList) {
	"use strict";
	var game = new Game(document.getElementById('main'));
	var ctx = game.context;

	function circle(x,y,radius) {
		ctx.beginPath();
		ctx.arc(x, y, radius, 0, Math.PI*2, true);
		ctx.closePath();
	}
	function rectangle(x,y,w,h) {
		ctx.rect(x,y,w,h);
	}

	function strokeRectangle(x,y,w,h) {
		ctx.strokeRect(x,y,w,h);
	}

	function fillRectangle(x,y,w,h) {
		ctx.fillRect(x,y,w,h);
	}

	function strokeCircle(x,y,radius) {
		circle(x,y,radius);
		ctx.stroke();
	}

	function fillCircle(x,y,radius) {
		circle(x,y,radius);
		ctx.fill();
	}

	var GameObject = Class({
		initGameObject: function() {
			this.position = new Vector(0,0);
		}
	});

	var pendingAddList = new IntrusiveList('pendingAdd');
	var pendingRemoveList = new IntrusiveList('pendingRemove');
	var gameObjectList = new DIntrusiveList('gameObject');
	var updateList = new DIntrusiveList('update');
	var drawList = new DIntrusiveList('draw');
	var collisionList = new DIntrusiveList('collision');
	var boundingList = new DIntrusiveList('bounding');
	var namedObjects = {};
	var gameObjectManager = {
		add: function(o) {
			pendingAddList.push(o);
		},
		remove: function(o) {
			pendingRemoveList.push(o);
		},
		handlePending: function() {
			pendingRemoveList.each(function(o,_,DELETE) {
				try {
					gameObjectList.remove(o);
					if (o.update) { updateList.remove(o); }
					if (o.draw) { drawList.remove(o); }
					if (o.collisionRadius) { collisionList.remove(o); }
					if (o.getBounds) { boundingList.remove(o); }
					if (o.name) { delete namedObjects[o.name]; }
				} catch(e) {
					console.error('Could not remove object',o,'Reason:',e);
				}
				return DELETE;
			});
			pendingAddList.each(function(o,_,DELETE) {
				try {
					gameObjectList.push(o);
					if (o.update) { updateList.push(o); }
					if (o.draw) { drawList.push(o); }
					if (o.collisionRadius) { collisionList.push(o); }
					if (o.getBounds) { boundingList.push(o); }
					if (o.name) {
						if (namedObjects[o.name]) { throw 'Duplicate object with name '+o.name; }
						namedObjects[o.name] = o;
					}
				} catch(e) {
					console.error('Could not add object',o,'Reason:',e);
				}
				return DELETE;
			});
		},
		update: function(dt) {
			updateList.each(function(o) {
				o.update(dt);
			});
			this.handlePending();
		}
	};

	function loadImage(name) {
		var img = new Image();
		img.src = name+'.png';
		return img;
	}

	function loadAnimation(name,sizex,sizey,frames) {
		var image = loadImage(name);
		var loaded = false;
		image.onload = function() { loaded=true; };

		return {
			frame: 0,
			rate: 1,
			update: function(dt) {
				var rate = this.rate;
				this.frame += dt*rate;
				while (this.frame > rate*frames) { this.frame -= rate*frames; }
			},
			draw: function(ctx,dx,dy,dw,dh) {
				if (loaded) {
					var frame = Math.floor(this.frame);
					try{
						ctx.drawImage(image,sizex*frame,0,sizex,sizey,dx,dy,dw,dh);
					} catch(e) {
						console.error(frame);
					}
				}
			}
		};
	}

	var Ball = Class(GameObject, {
		constructor: function Ball() {
			this.initGameObject();
			this.velocity = new Vector(0,0);
			this.gravity = 0;
			this.collisionRadius = 20;
			this.anim = loadAnimation('mafkees',42,48,3);
		},
		update: function(dt) {
			this.position.add(this.velocity.x*dt, this.velocity.y*dt);

			this.anim.update(dt*Math.abs(this.velocity.x)/30);
			if (Math.abs(this.velocity.x) < 3) {
				this.anim.frame = 0;
			}
		},
		draw: function(ctx) {
			ctx.save();
			ctx.translate(this.position.x,this.position.y);
			ctx.scale(this.velocity.x < 0 ? -1 : 1,1);
			ctx.translate(-this.position.x,-this.position.y);
			this.anim.draw(ctx,this.position.x-21,this.position.y-27,42,48);
			ctx.restore();
		},
		shoot: function() {
			var b = new Bullet(1000,0);
			b.position.setV(this.position);
			gameObjectManager.add(b);
		},
		jump: function() {
			if (this.surface && this.surface.normaly < 0) {
				this.velocity.add(this.surface.normalx*800,this.surface.normaly*800);
			}
		}
	});


	var Bullet = Class(GameObject, {
		constructor: function Bullet(vx,vy) {
			this.initGameObject();
			this.velocity = new Vector(vx||0,vy||0);
			this.lifeTime = 1;
		},
		update: function(dt) {
			this.position.add(this.velocity.x*dt, this.velocity.y*dt);
			this.lifeTime -= dt;
			if(this.lifeTime < 0) {
				gameObjectManager.remove(this);
			}
		},
		draw: function(ctx) {
			strokeCircle(this.position.x, this.position.y, 10);
		}
	});
	var Apple = Class(GameObject, {
		constructor: function Apple(x,y) {
			this.initGameObject();
			this.position.set(x,y);
		},
		draw: function(ctx) {
			ctx.fillColor = 'red';
			fillCircle(this.position.x, this.position.y, 10);
			ctx.fillColor = 'black';
		}
	});

	var Beam = Class(GameObject, {
		initBeam: function Beam(x,y,w,h,name) {
			this.initGameObject();
			this.position.set(x,y);
			this.bounds = Room.Rectangle(x,y,w,h);
			this.img = loadImage(name);
		},
		getBounds: function() {
			return this.bounds;
		},
		draw: function(ctx) {
			ctx.drawImage(this.img,this.position.x-4,this.position.y-4);
		}
	});

	var Beam100 = Class(Beam, {
		constructor: function Beam100(x,y) {
			this.initBeam(x,y,100,20,'beam100');
		}
	});
	var Beam200 = Class(Beam, {
		constructor: function Beam200(x,y) {
			this.initBeam(x,y,200,20,'beam200');
		}
	});
	var Beam300 = Class(Beam, {
		constructor: function Beam300(x,y) {
			this.initBeam(x,y,300,20,'beam300');
		}
	});
	var Block100 = Class(Beam, {
		constructor: function Block100(x,y) {
			this.initBeam(x,y,100,100,'block100');
		}
	});

	var Door = Class(GameObject, {
		constructor: function Door(x,y,w,h,openf,name) {
			this.initGameObject();
			this.initDoor(x,y,w,h,openf);
			this.name = name;
			this.img = loadImage('door');
		},
		initDoor: function(x,y,w,h,openf) {
			this.position.set(x,y);
			this.width = w;
			this.height = h;
			this.bounds = Room.Rectangle(x,y,w,h);
			if (typeof openf === 'boolean') { var openValue = openf; openf = function() { return openValue; }; }
			this.openf = openf || function() { return false; };
		},
		getBounds: function() {
			if (this.openf()) {
				return [];
			} else {
				return this.bounds;
			}
		},
		draw: function(ctx) {
			if (!this.openf()) {
				var rotate = 0;
				if (this.width < this.height) { rotate = Math.PI/2; }
				var mx = this.position.x+this.width/2;
				var my = this.position.y+this.height/2;
				var x = mx-44;
				var y = my-9;
				ctx.save();
				ctx.translate(mx,my);
				ctx.rotate(rotate);
				ctx.translate(-mx,-my);
				ctx.drawImage(this.img,x, y-4);
				ctx.restore();
			}
		}
	});

	var Spikes = Class(GameObject, {
		constructor: function(x,y) {
			this.initGameObject();
			this.position.set(x,y);
			this.img = loadImage('spikes');
		},
		update: function(dt) {
			if (this.position.distanceToV(ball.position) < 100) {
				die();
			}
		},
		draw: function(ctx) {
			ctx.drawImage(this.img, this.position.x-217/2,this.position.y-140);
		}
	});

	var Other = Class(GameObject, {
		constructor: function(x,y) {
			this.initGameObject();
			this.position.set(x,y);
			this.velocity = new Vector(0,0);
			this.img = loadImage('other');
			this.state = 'looking';
			this.collisionRadius = 20;
		},
		update: function(dt) {
			var me=this;
			({
				looking: function() {
					var d = me.position.distanceToV(ball.position);
					if (d < 90) {
						me.state = 'falling';
						me.velocity.x += 280;
						me.velocity.y -= 20;
					}
				},
				falling: function() {
					me.velocity.add(-0,9.81*250*dt);
					me.position.add(me.velocity.x*dt, me.velocity.y*dt);
					gameObjectList.each(function(o) {
						if (o.constructor === Spikes) {
							if (me.position.distanceToV(o.position) < 100) {
								gameObjectManager.remove(me);
								spawnBlood(me.position.x,me.position.y);
								var c = new Corpse(me.position.x,me.position.y,'othercorpse');
								c.velocity.setV(me.velocity);
								gameObjectManager.add(c);
							}
						}
					});
				}
			}[this.state]());
		},
		draw: function(ctx) {
			var me=this;
			({
				looking: function() {
					ctx.drawImage(me.img,0,0,42,48,me.position.x-21,me.position.y-24,42,48);
				},
				falling: function() {
					ctx.drawImage(me.img,42,0,42,48,me.position.x-21,me.position.y-24,42,48);
				}
			}[this.state]());
		}
	});

	var Sign = Class(GameObject, {
		constructor: function(x,y,imgName) {
			this.initGameObject();
			this.position.set(x,y);
			this.img = loadImage(imgName);
		},
		draw: function(ctx) {
			if (this.img.complete) {
				ctx.drawImage(this.img,this.position.x-this.img.width/2,this.position.y-this.img.height/2);
			}
		}
	});

	var Button = Class(GameObject, Event, {
		pressedf: null,
		constructor: function Button() { },
		initButton: function(x,y,pressedf) {
			this.initGameObject();
			this.position.set(x,y);
			this.size = 20;
			this.pressedf = pressedf;
			this.img = loadImage('weightbutton');
		},
		update: function(dt) {
			var me = this;
			var pressed = this.pressedf();
			if (this.pressed !== pressed) {
				this.pressed = pressed;
				if (pressed) {
					this.emit('pressed');
				} else {
					this.emit('unpressed');
				}
				this.emit('toggled');
			}
		},
		draw: function(ctx) {
			ctx.drawImage(this.img,0,this.pressed?6:0,40,6,this.position.x-20,this.position.y-6,40,6);
		}
	});


	var WeightButton = Class(Button, {
		constructor: function WeightButton(x,y) {
			var me = this;
			this.initButton(x,y,function() {
				var pressed = false;
				collisionList.each(function(o) {
					pressed = pressed || o.position.distanceToV(me.position) < o.collisionRadius+me.size;
				});
				return pressed;
			});
		}
	});

	var TimedButton = Class(Button, {
		constructor: function TimedButton(x,y) {
			var pressed = false;
			var timeout = null;
			var me = this;
			this.initButton(x,y,function() {
				var pressing;
				collisionList.each(function(o) {
					pressing = pressing || o.position.distanceToV(me.position) < o.collisionRadius+me.size;
				});
				if (pressing) {
					pressed = true;
					clearTimeout(timeout);
					setTimeout(function() { pressed=false; }, 3000);
				}
				return pressed;
			});
		}
	});

	var Item = Class(GameObject, {
		constructor: function Item(x,y,img) {
			this.initGameObject();
			this.initialPosition = new Vector(x,y);
			this.position.set(x,y);
			this.collisionRadius = 10;
			this.velocity = new Vector(0,0);
			this.canPickup = true;
			this.img = loadImage(img);
		},
		update: function(dt) {
			var me = this;
			if (this.surface) {
				this.velocity.multiply(0.4);
			} else {
				this.velocity.multiply(0.95);
			}
			this.velocity.y += 9.81*dt*300;
			var v = new Vector(0,0);
			collisionList.each(function(o) {
				if (o === me) { return; }
				v.setV(me.position);
				v.substractV(o.position);
				var dist = v.length();
				var d = o.collisionRadius+me.collisionRadius;
				if (dist < d) {
					v.normalizeOrZero();
					v.multiply(100);
					o.velocity.add(-v.x,-v.y);
					me.velocity.add(v.x,v.y);
				}
			});
			this.position.add(this.velocity.x*dt, this.velocity.y*dt);
			if (this.position.x < 0 || this.position.x > Room.size || this.position.y < 0 || this.position.y > Room.size) {
				this.position.setV(this.initialPosition);
				this.velocity.set(0,0);
			}
		},
		draw: function(ctx) {
			ctx.drawImage(this.img,this.position.x-16,this.position.y-16,32,32);
		}
	});

	var Switch = Class(GameObject, {
		constructor: function(x,y) {
			this.initGameObject();
			this.position.set(x,y);
			this.img = loadImage('switch');
			this.collisionRadius = 5;
			this.pressed = false;
		},
		use: function() {
			this.pressed = !this.pressed;
		},
		draw: function(ctx) {
			ctx.drawImage(this.img,0,this.pressed?0:27,27,27, this.position.x,this.position.y-13,27,27);
		}
	});

	var Corpse = Class(Item, {
		constructor: function(x,y,imgName) {
			this.initGameObject();
			this.position.set(x,y);
			this.img = loadImage(imgName || 'corpse');
			this.collisionRadius = 20;
			this.canPickup = true;
			this.velocity = new Vector(0,0);
		},
		draw: function(ctx) {
			ctx.drawImage(this.img,this.position.x-24,this.position.y-20);
		}
	});

	var Blood = Class(GameObject, {
		constructor: function(x,y,vx,vy) {
			this.initGameObject();
			this.position.set(x,y);
			this.velocity = new Vector(vx,vy);
			this.img = loadImage('blood');
			this.time = 3;
			this.imgIndex = Math.floor(Math.random()*4);
		},
		update: function(dt) {
			this.time -= dt;
			if (this.time < 0) {
				gameObjectManager.remove(this);
				return;
			}
			this.velocity.multiply(0.98);
			this.velocity.y += 9.81*dt*100;
			this.position.add(this.velocity.x*dt,this.velocity.y*dt);
		},
		draw: function(ctx) {
			var ix = (this.imgIndex%2)*12;
			var iy = Math.floor(this.imgIndex/2)*12;
			ctx.drawImage(this.img,ix,iy,12,12,this.position.x-6,this.position.y-6,12,12);
		}
	});

	Room.Spikes = Spikes;
	Room.WeightButton = WeightButton;
	Room.TimedButton = TimedButton;
	Room.Door = Door;
	Room.Item = Item;
	Room.Beam100 = Beam100;
	Room.Beam200 = Beam200;
	Room.Beam300 = Beam300;
	Room.Block100 = Block100;
	Room.Switch = Switch;
	Room.Other = Other;
	Room.Sign = Sign;

	Room.AllPressedOnce = function(/*...*/) {
		var pressed = false;
		var buttons = arguments;
		for(var i=0;i<buttons.length;i++) {
			buttons[i].on('pressed', function() {
				var p=true;
				for(var j=0;j<buttons.length;j++) {
					p = p && buttons[j].pressed;
				}
				if (p) {
					pressed = true;
				}
			});
		}
		return function() { return pressed; };
	};
	Room.AllPressed = function(/* ... */) {
		var buttons = arguments;
		return function() {
			var p = true;
			for(var i=0;i<buttons.length;i++) {
				p = p && buttons[i].pressed;
			}
			return p;
		};
	};

	Room.DoorNorth = function(open) {
		return new Door(Room.size/2-Room.doorSize/2,30,Room.doorSize,20,open,'doorN');
	};
	Room.DoorSouth = function(open) {
		return new Door(Room.size/2-Room.doorSize/2,Room.size-40,Room.doorSize,20,open,'doorS');
	};
	Room.DoorWest = function(open) {
		return new Door(28,Room.size/2-Room.doorSize/2,10,Room.doorSize,open,'doorW');
	};
	Room.DoorEast = function(open) {
		return new Door(Room.size-45,Room.size/2-Room.doorSize/2,10,Room.doorSize,open,'doorE');
	};
	Room.Doors = function(n,w,e,s) {
		return [Room.DoorNorth(n),Room.DoorWest(w),Room.DoorEast(e),Room.DoorSouth(s)];
	};


	function navigateTo(direction) {
		if (!room) { throw 'how the hell did i get here?'; }
		var roomName = room.exits[direction];
		if (!roomName) {
			console.error('No valid exit!');
			return;
		}
		enterRoom(roomName,{
			N:'S',
			S:'N',
			E:'W',
			W:'E'
		}[direction]);
	}

	function placeAtEntrance(direction) {
		var off = 5;
		ball.position.setV({
			N:new Vector(game.width/2,off),
			S:new Vector(game.width/2,game.height-off),
			W:new Vector(off,game.height/2),
			E:new Vector(game.width-off,game.height/2)
		}[direction]);
		if (direction === 'S') {
			ball.velocity.y = Math.min(-700, ball.velocity.y-200);
		}
	}

	function enterRoom(roomName,direction) {
		if (room) {
			gameObjectManager.handlePending();
			var gol = [];
			gameObjectList.each(function(o) {
				if (o !== ball) {
					gameObjectManager.remove(o);
					gol.push(o);
				}
			});
			gameObjectManager.handlePending();
			room.objects = gol;
			room = null;
		}
		if (direction) {
			startEntrance = direction;
			placeAtEntrance(direction);
		}
		game.stop();
		require(['rooms/'+roomName], function(loadedRoom) {
			room = loadedRoom;
			room.name = roomName;
			var os = room.objects;
			for(var i=0;i<os.length;i++) {
				gameObjectManager.add(os[i]);
			}
			gameObjectManager.handlePending();

			game.start();
		});
	}


	var died = 0;
	var startEntrance = null;
	var ball = new Ball();
	ball.position.set(320,320);
	gameObjectManager.add(ball);

	function die() {
		console.log('die');
		if (died > 0) { return; }
		died = 3;
		ball.velocity.set(0,0);
		gameObjectManager.remove(ball);
		var c = new Corpse(ball.position.x,ball.position.y);
		gameObjectManager.add(c);

		spawnBlood(ball.position.x, ball.position.y);

		gameObjectManager.handlePending();
		placeAtEntrance(startEntrance);
		console.log('place at',startEntrance);
	}

	function spawnBlood(x,y) {
		var m = 500;
		function r() { return (Math.random()-0.5)*2; }
		for(var i=0;i<10;i++) {
			var b = new Blood(x+r()*5, y+r()*5, r()*m, r()*m);
			gameObjectManager.add(b);
		}
	}

	var room = null;
	game.on('keydown', function(key) {
		if (key === 's') {
			if (ball.pickup) {
				gameObjectManager.add(ball.pickup);
				var dir = ball.velocity.x < 0 ? -1 : 1;
				ball.pickup.position.set(ball.position.x+dir*35,ball.position.y);
				ball.pickup.velocity.setV(ball.velocity);
				ball.pickup = null;
			} else {
				collisionList.each(function(o,BREAK) {
					if (ball.position.distanceToV(o.position) < ball.collisionRadius+o.collisionRadius+10) {
						if (o.canPickup) {
							gameObjectManager.remove(o);
							ball.pickup = o;
							return BREAK;
						} else if (o.use) {
							o.use();
						}
					}
				});
			}
		}
	});
	game.on('update', function(dt) {
		if (died > 0) {
			died -= dt;
			if (died <= 0) {
				died = 0;
				gameObjectManager.add(ball);
				gameObjectManager.handlePending();
				placeAtEntrance(startEntrance);
				ball.velocity.set(0,0);
				console.log('reset');
				return;
			}
		}

		var movement = new Vector(
			(game.keys.right ? 1 : 0) - (game.keys.left ? 1 : 0),
			(game.keys.down ? 1 : 0) - (game.keys.up ? 1 : 0)
		);

		if (ball.surface && ball.surface.normaly < 0) {
			ball.velocity.x *= 0.70;
		} else {
			ball.velocity.x *= 0.98;
		}
		ball.velocity.x = Math.min(300,Math.max(-300, ball.velocity.x + movement.x*100));
		ball.velocity.y += 9.81*dt*250;

		if (game.keys.a) {
			ball.jump();
		}

		if (ball.pickup) {
			var dir = ball.velocity.x < 0 ? -1 : 1;
			dir*=5;
			ball.pickup.position.set(ball.position.x+dir,ball.position.y);
		}

		gameObjectManager.update(dt);


		(function handleCollision() {
			var t = new Vector(0,0)
			var t2 = new Vector(0,0);
			collisionList.each(function(o) {
				if (!o.velocity){return;}
				var lineSegments = room.lineSegments;
				var collisions = [];
				function handleCollisionLineSegments(lineSegments) {
					for(var i=0;i<lineSegments.length;i++) {
						var lineSegment = lineSegments[i];
						t.setV(lineSegment.normal);
						t.normalRight();
						var l = lineSegment.start.distanceToV(lineSegment.end);
						t2.setV(o.position);
						t2.substractV(lineSegment.start);
						var offY = lineSegment.normal.dotV(t2)-o.collisionRadius;
						var offX = t.dotV(t2);
						if (offY < -o.collisionRadius*2) {
							continue;
						} else if (offY < 0) {
							if (offX > 0 && offX < l) {
								offY*=-1;
								collisions.push({
									normalx:lineSegment.normal.x,
									normaly:lineSegment.normal.y,
									offset:offY
								});
							} else if (offX < 0 && offX > -o.collisionRadius) {
								var d = o.position.distanceToV(lineSegment.start);
								if (d < o.collisionRadius) {
									t.setV(o.position);
									t.substractV(lineSegment.start);
									t.normalize();
									collisions.push({
										normalx:t.x,
										normaly:t.y,
										offset:o.collisionRadius-d
									});
								}
							} else if (offX > l && offX < l+o.collisionRadius) {
								var d = o.position.distanceToV(lineSegment.end);
								if (d < o.collisionRadius) {
									t.setV(o.position);
									t.substractV(lineSegment.end);
									t.normalize();
									collisions.push({
										normalx:t.x,
										normaly:t.y,
										offset:o.collisionRadius-d
									});
								}
							}
						} else {
							continue;
						}
					}
				}
				handleCollisionLineSegments(lineSegments);
				boundingList.each(function(bounder) {
					var lineSegments = bounder.getBounds();
					handleCollisionLineSegments(lineSegments);
				});
				if (collisions.length > 0) {
					collisions.sort(function(a,b) {
						return b.offset-a.offset;
					});
					var c = collisions[0];
					o.position.add(c.normalx*c.offset,c.normaly*c.offset);
					var vc = o.velocity.dot(c.normalx, c.normaly);
					o.velocity.substract(c.normalx*vc, c.normaly*vc);
					o.surface = c;
				} else {
					o.surface = null;
				}
			});
		}());

		if (ball.position.x < 0) {
			navigateTo('W');
		} else if (ball.position.x > game.width) {
			navigateTo('E');
		} else if (ball.position.y < 0) {
			navigateTo('N');
		} else if (ball.position.y > game.height) {
			navigateTo('S');
		}

	});

	var borderImage = loadImage('borders');
	game.on('draw', function(ctx) {
		drawList.each(function(o) {
			o.draw(ctx);
		});
		ctx.drawImage(borderImage,0,0,640,640);

		if (ball.pickup) {
			ball.pickup.draw(ctx);
		}
	});

	function createEditor() {
		var mode=null;
		var round=10;
		function r(f) {
			return Math.floor(f/round)*round;
		}
		game.on('mousedown', function() {
			if (mode && mode.leftdown)
				mode.leftdown(new Vector(r(game.mouse.x),r(game.mouse.y)));
		});
		game.on('mouseup', function() {
			if (mode && mode.leftup)
				mode.leftup(new Vector(r(game.mouse.x),r(game.mouse.y)));
		});

		function createButton(text,onclick) {
			var button = document.createElement('input');
			button.setAttribute('type','button');
			button.setAttribute('value',text);
			button.onclick = onclick;
			document.body.appendChild(button);
		}

		function deleteLine(p) {
			var v = new Vector(0,0);
			for(var i=0;i<room.lineSegments.length;i++) {
				var ls = room.lineSegments[i];
				v.setV(ls.start);
				v.addV(ls.end);
				v.divide(2);
				var dst = v.distanceToV(p);
				if (dst < 5) {
					room.lineSegments.splice(i,1);
					return;
				}
			}
		}

		function pickObject(p) {
			var r = null;
			boundingList.each(function(o,BREAK) {
				var lineSegments = o.getBounds();

				var inside = lineSegments.length > 0;
				var k = new Vector(0,0);
				for(var i=0;i<lineSegments.length;i++) {
					k.setV(p);
					k.substractV(lineSegments[i].start);
					inside = inside && (lineSegments[i].normal.dotV(k) <= 0);
				}
				if (inside) {
					r = o;
					return BREAK;
				}
			});
			return r;
		}
		function moveBounds(o,v) {
			if (!o.bounds) return;
			o.position.addV(v);
			for(var i=0;i<o.bounds.length;i++) {
				o.bounds[i].start.addV(v);
				o.bounds[i].end.addV(v);
			}
		}
		createButton('move', function() {
			var picked = null;
			var startp;
			mode = {
				leftdown: function(p) {
					startp = p;
					picked = pickObject(p);
				},
				leftup: function(p) {
					if (picked) {
						p.substractV(startp);
						moveBounds(picked,p);
					}
				}
			};
		});

		createButton('delete', function() {
			mode = {
				leftdown: function(p) {
					var p = pickObject(p);
					gameObjectManager.remove(p);
				}
			};
		});

		function createObjectCreateButton(name) {
			createButton(name, function() {
				mode = {
					leftdown: function(p) {
						var constructor = eval(name);
						var obj = new constructor(p.x,p.y);
						gameObjectManager.add(obj);
					}
				};
			});
		}
		createObjectCreateButton('Beam100');
		createObjectCreateButton('Beam200');
		createObjectCreateButton('Beam300');
		createObjectCreateButton('Block100');

		createButton('serialize', function() {
			var r = '';
			boundingList.each(function(o) {
				var p = ['Room.Beam100','Room.Beam200','Room.Beam300','Room.Block100'];
				var name = null;
				for(var i=0;i<p.length;i++) {
					var constructor = eval(p[i]);
					if (o.constructor === constructor) {
						name = p[i];
					}
				}
				if (name) {
					r += ',new '+name+'('+o.position.x+','+o.position.y+')';
				}
			});
			console.log(r);
		});
		createButton('enter', function() {
			var name = prompt('Room:', room.name);
			if (name) {
				enterRoom(name);
			}
		});
	}

	enterRoom('0_1');

});
